@@ -14,6 +14,7 @@ import sys
 import threading
 import socket
 import warnings
+import ssl
 from datetime import datetime
 
 from urllib3.exceptions import HTTPWarning
@@ -30,6 +31,20 @@ CERTS_PATH = os.path.join(os.path.dirname(__file__), 'certs')
 DEFAULT_CERTS = {
     'certfile': os.path.join(CERTS_PATH, 'server.crt'),
     'keyfile': os.path.join(CERTS_PATH, 'server.key'),
+    'cert_reqs': ssl.CERT_OPTIONAL,
+    'ca_certs': os.path.join(CERTS_PATH, 'cacert.pem'),
+}
+DEFAULT_CLIENT_CERTS = {
+    'certfile': os.path.join(CERTS_PATH, 'client_intermediate.pem'),
+    'keyfile': os.path.join(CERTS_PATH, 'client_intermediate.key'),
+    'subject': dict(countryName=u'FI', stateOrProvinceName=u'dummy',
+                    organizationName=u'dummy', organizationalUnitName=u'dummy',
+                    commonName=u'SnakeOilClient',
+                    emailAddress=u'dummy@test.local'),
+}
+DEFAULT_CLIENT_NO_INTERMEDIATE_CERTS = {
+    'certfile': os.path.join(CERTS_PATH, 'client_no_intermediate.pem'),
+    'keyfile': os.path.join(CERTS_PATH, 'client_intermediate.key'),
 }
 NO_SAN_CERTS = {
     'certfile': os.path.join(CERTS_PATH, 'server.no_san.crt'),
@@ -48,6 +63,7 @@ DEFAULT_CA_BAD = os.path.join(CERTS_PATH, 'client_bad.pem')
 NO_SAN_CA = os.path.join(CERTS_PATH, 'cacert.no_san.pem')
 DEFAULT_CA_DIR = os.path.join(CERTS_PATH, 'ca_path_test')
 IPV6_ADDR_CA = os.path.join(CERTS_PATH, 'server.ipv6addr.crt')
+COMBINED_CERT_AND_KEY = os.path.join(CERTS_PATH, 'server.combined.pem')
 
 
 def _has_ipv6(host):
@@ -65,7 +81,7 @@ def _has_ipv6(host):
             sock = socket.socket(socket.AF_INET6)
             sock.bind((host, 0))
             has_ipv6 = True
-        except:
+        except Exception:
             pass
 
     if sock:
@@ -210,13 +226,16 @@ def bind_sockets(port, address=None, family=socket.AF_UNSPEC, backlog=128,
 
 
 def run_tornado_app(app, io_loop, certs, scheme, host):
-    app.last_req = datetime.fromtimestamp(0)
+    assert io_loop == tornado.ioloop.IOLoop.current()
+
+    # We can't use fromtimestamp(0) because of CPython issue 29097, so we'll
+    # just construct the datetime object directly.
+    app.last_req = datetime(1970, 1, 1)
 
     if scheme == 'https':
-        http_server = tornado.httpserver.HTTPServer(app, ssl_options=certs,
-                                                    io_loop=io_loop)
+        http_server = tornado.httpserver.HTTPServer(app, ssl_options=certs)
     else:
-        http_server = tornado.httpserver.HTTPServer(app, io_loop=io_loop)
+        http_server = tornado.httpserver.HTTPServer(app)
 
     sockets = bind_sockets(None, address=host)
     port = sockets[0].getsockname()[1]
@@ -250,7 +269,7 @@ if __name__ == '__main__':
     from .testcase import TestingApp
     host = '127.0.0.1'
 
-    io_loop = tornado.ioloop.IOLoop()
+    io_loop = tornado.ioloop.IOLoop.current()
     app = tornado.web.Application([(r".*", TestingApp)])
     server, port = run_tornado_app(app, io_loop, None,
                                    'http', host)
